"use strict";
/*!
 * Copyright (c) Microsoft Corporation and contributors. All rights reserved.
 * Licensed under the MIT License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.describeInstallVersions = void 0;
const test_utils_1 = require("@fluidframework/test-utils");
const compatOptions_1 = require("./compatOptions");
const compatUtils_1 = require("./compatUtils");
const packageVersion_1 = require("./packageVersion");
const testApi_1 = require("./testApi");
const installRequiredVersions = async (config) => {
    const installPromises = [];
    if (config.requestAbsoluteVersions !== undefined) {
        installPromises.push(...config.requestAbsoluteVersions.map(async (version) => (0, testApi_1.ensurePackageInstalled)(version, 0, /* force */ false)));
    }
    if (config.requestRelativeVersions !== undefined) {
        installPromises.push((0, testApi_1.ensurePackageInstalled)(packageVersion_1.pkgVersion, config.requestRelativeVersions, /* force */ false));
    }
    let hadErrors = false;
    for (const promise of installPromises) {
        try {
            await promise;
        }
        catch (e) {
            console.error(e);
            hadErrors = true;
        }
    }
    if (hadErrors) {
        throw new Error("Exceptions while installing package versions. Check STDERR");
    }
};
const defaultTimeoutMs = 20000;
const defaultRequestedVersions = { requestRelativeVersions: -2 };
function createTestSuiteWithInstalledVersion(tests, requiredVersions = defaultRequestedVersions, timeoutMs = defaultTimeoutMs) {
    return function () {
        let defaultProvider;
        let resetAfterEach;
        before(async function () {
            this.timeout(Math.max(defaultTimeoutMs, timeoutMs));
            await installRequiredVersions(requiredVersions);
            defaultProvider = await (0, compatUtils_1.getVersionedTestObjectProvider)(packageVersion_1.pkgVersion, // baseVersion
            packageVersion_1.pkgVersion, // loaderVersion
            {
                type: compatOptions_1.driver,
                version: packageVersion_1.pkgVersion,
                config: {
                    r11s: { r11sEndpointName: compatOptions_1.r11sEndpointName },
                    odsp: { tenantIndex: compatOptions_1.tenantIndex },
                },
            }, // driverConfig
            packageVersion_1.pkgVersion, // runtimeVersion
            packageVersion_1.pkgVersion);
            Object.defineProperty(this, "__fluidTestProvider", { get: () => defaultProvider });
        });
        tests.bind(this)((options) => {
            var _a;
            resetAfterEach = (_a = options === null || options === void 0 ? void 0 : options.resetAfterEach) !== null && _a !== void 0 ? _a : true;
            if ((options === null || options === void 0 ? void 0 : options.syncSummarizer) === true) {
                defaultProvider.resetLoaderContainerTracker(true /* syncSummarizerClients */);
            }
            return defaultProvider;
        });
        afterEach(function (done) {
            var _a;
            const logErrors = (0, test_utils_1.getUnexpectedLogErrorException)(defaultProvider.logger);
            // if the test failed for another reason
            // then we don't need to check errors
            // and fail the after each as well
            if (((_a = this.currentTest) === null || _a === void 0 ? void 0 : _a.state) === "passed") {
                done(logErrors);
            }
            else {
                done();
            }
            if (resetAfterEach) {
                defaultProvider.reset();
            }
        });
    };
}
/**
 * Creates a test suite which will priorly install a set of requested Fluid versions for the tests to use.
 * The packages are installed before any test code runs, so it is guaranteed that the package is present
 * when the test code is invoked, including the top level scope inside the `describeInstallVersions` block.
 *
 * If package installation fails for any of the requested versions, the test suite will not be created and
 * the test run will fail.
 *
 * @param requestedVersions - See {@link IRequestedFluidVersions}.
 * If unspecified, the test will install the last 2 versions.
 * @param timeoutMs - the timeout for the tests in milliseconds, as package installation is time consuming.
 * If unspecified, the timeout is 20000 ms.
 * @returns A mocha test suite
 */
function describeInstallVersions(requestedVersions, timeoutMs) {
    const d = (name, tests) => describe(name, createTestSuiteWithInstalledVersion(tests, requestedVersions, timeoutMs));
    d.skip = (name, tests) => describe.skip(name, createTestSuiteWithInstalledVersion(tests, requestedVersions, timeoutMs));
    d.only = (name, tests) => describe.only(name, createTestSuiteWithInstalledVersion(tests, requestedVersions, timeoutMs));
    return d;
}
exports.describeInstallVersions = describeInstallVersions;
//# sourceMappingURL=describeWithVersions.js.map